//===== Copyright  1996-2005, Valve Corporation, All rights reserved. ======//
//
// Purpose: 
//
// $NoKeywords: $
//
//===========================================================================//

#include <stdio.h>

#include "interface.h"
#include "windows.h"
#include "filesystem.h"
#include "engine/iserverplugin.h"
#include "game/server/iplayerinfo.h"
#include "eiface.h"
#include "igameevents.h"
#include "convar.h"
#include "Color.h"
#include "vstdlib/random.h"
#include "engine/IEngineTrace.h"
#include "tier2/tier2.h"
#include <map>

#include "common/GMLuaModule.h"

GMOD_MODULE(Start, Close);

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

// Interfaces from the engine
IVEngineServer	*engine = NULL; // helper functions (messaging clients, loading content, making entities, running commands, etc)
IGameEventManager *gameeventmanager = NULL; // game events interface
IPlayerInfoManager *playerinfomanager = NULL; // game dll interface to interact with players
IBotManager *botmanager = NULL; // game dll interface to interact with bots
IServerPluginHelpers *helpers = NULL; // special 3rd party plugin helpers from the engine
IUniformRandomStream *randomStr = NULL;
IEngineTrace *enginetrace = NULL;
IFileSystem *filesystem = NULL;


CGlobalVars *gpGlobals = NULL;

//---------------------------------------------------------------------------------
// Purpose: a sample 3rd party plugin class
//---------------------------------------------------------------------------------
class CEmptyServerPlugin: public IServerPluginCallbacks, public IGameEventListener
{
public:
	CEmptyServerPlugin();
	~CEmptyServerPlugin();

	// IServerPluginCallbacks methods
	virtual bool			Load(	CreateInterfaceFn interfaceFactory, CreateInterfaceFn gameServerFactory );
	virtual void			Unload( void );
	virtual void			Pause( void );
	virtual void			UnPause( void );
	virtual const char     *GetPluginDescription( void );      
	virtual void			LevelInit( char const *pMapName );
	virtual void			ServerActivate( edict_t *pEdictList, int edictCount, int clientMax );
	virtual void			GameFrame( bool simulating );
	virtual void			LevelShutdown( void );
	virtual void			ClientActive( edict_t *pEntity );
	virtual void			ClientDisconnect( edict_t *pEntity );
	virtual void			ClientPutInServer( edict_t *pEntity, char const *playername );
	virtual void			SetCommandClient( int index );
	virtual void			ClientSettingsChanged( edict_t *pEdict );
	virtual PLUGIN_RESULT	ClientConnect( bool *bAllowConnect, edict_t *pEntity, const char *pszName, const char *pszAddress, char *reject, int maxrejectlen );
	virtual PLUGIN_RESULT	ClientCommand( edict_t *pEntity, const CCommand &args );
	virtual PLUGIN_RESULT	NetworkIDValidated( const char *pszUserName, const char *pszNetworkID );
	virtual void			OnQueryCvarValueFinished( QueryCvarCookie_t iCookie, edict_t *pPlayerEntity, EQueryCvarValueStatus eStatus, const char *pCvarName, const char *pCvarValue );

	// IGameEventListener Interface
	virtual void FireGameEvent( KeyValues * event );
};

CEmptyServerPlugin g_EmptyServerPlugin;
EXPOSE_SINGLE_INTERFACE_GLOBALVAR(CEmptyServerPlugin, IServerPluginCallbacks, INTERFACEVERSION_ISERVERPLUGINCALLBACKS, g_EmptyServerPlugin );

CEmptyServerPlugin::CEmptyServerPlugin()
{

}

CEmptyServerPlugin::~CEmptyServerPlugin()
{
}

bool CEmptyServerPlugin::Load(	CreateInterfaceFn interfaceFactory, CreateInterfaceFn gameServerFactory )
{
	ConnectTier1Libraries( &interfaceFactory, 1 );
	ConnectTier2Libraries( &interfaceFactory, 1 );

	engine = (IVEngineServer*)interfaceFactory(INTERFACEVERSION_VENGINESERVER, NULL);

	filesystem = (IFileSystem *)interfaceFactory(FILESYSTEM_INTERFACE_VERSION, NULL);

	MathLib_Init( 2.2f, 2.2f, 0.0f, 2.0f );
	ConVar_Register( 0 );
	return true;
}

void CEmptyServerPlugin::Unload( void )
{
	ConVar_Unregister( );
	DisconnectTier2Libraries( );
	DisconnectTier1Libraries( );
}

void CEmptyServerPlugin::Pause( void )
{
}

void CEmptyServerPlugin::UnPause( void )
{
}

const char *CEmptyServerPlugin::GetPluginDescription( void )
{
	return "Mount Plugin";
}

void CEmptyServerPlugin::LevelInit( char const *pMapName )
{
}

void CEmptyServerPlugin::ServerActivate( edict_t *pEdictList, int edictCount, int clientMax )
{
}

void CEmptyServerPlugin::GameFrame( bool simulating )
{
}

void CEmptyServerPlugin::LevelShutdown( void )
{

}

void CEmptyServerPlugin::ClientActive( edict_t *pEntity )
{
}

void CEmptyServerPlugin::ClientDisconnect( edict_t *pEntity )
{
}

void CEmptyServerPlugin::ClientPutInServer( edict_t *pEntity, char const *playername )
{

}

void CEmptyServerPlugin::SetCommandClient( int index )
{
}

void CEmptyServerPlugin::ClientSettingsChanged( edict_t *pEdict )
{

}

PLUGIN_RESULT CEmptyServerPlugin::ClientConnect( bool *bAllowConnect, edict_t *pEntity, const char *pszName, const char *pszAddress, char *reject, int maxrejectlen )
{
	return PLUGIN_CONTINUE;
}

PLUGIN_RESULT CEmptyServerPlugin::ClientCommand( edict_t *pEntity, const CCommand &args )
{
	return PLUGIN_CONTINUE;
}

PLUGIN_RESULT CEmptyServerPlugin::NetworkIDValidated( const char *pszUserName, const char *pszNetworkID )
{
	return PLUGIN_CONTINUE;
}

void CEmptyServerPlugin::OnQueryCvarValueFinished( QueryCvarCookie_t iCookie, edict_t *pPlayerEntity, EQueryCvarValueStatus eStatus, const char *pCvarName, const char *pCvarValue )
{
}

void CEmptyServerPlugin::FireGameEvent( KeyValues * event )
{
}

std::map<int, bool> mountedApps;

void MountGame(ILuaInterface *gLua, const char *path, int appid)
{
	char buff[64];
	filesystem->AddSearchPath(path, "GAME");
	if(filesystem->MountSteamContent(-appid)==FILESYSTEM_MOUNT_FAILED)
	{
		sprintf(buff, "[NOT OK] Unable to mount appid %d", appid);
		gLua->Error(buff);
	} else {
		mountedApps[appid] = true;
		sprintf(buff, "[OK] Mounted appid %d\n", appid);
		gLua->Msg(buff);
	}
}

LUA_FUNCTION(IsGameMounted)
{
	ILuaInterface *gLua = Lua();

	gLua->CheckType(1, GLua::TYPE_NUMBER);

	if(mountedApps.find(gLua->GetInteger(1)) == mountedApps.end())
		gLua->Push(false);
	else
		gLua->Push(true);
	return 1;
}

LUA_FUNCTION(MountOtherGame)
{
	ILuaInterface *gLua = Lua();

	gLua->CheckType(1, GLua::TYPE_STRING);
	gLua->CheckType(2, GLua::TYPE_NUMBER);

	MountGame(gLua, gLua->GetString(1), gLua->GetInteger(2));

	//MountGame(gLua, "mm", -2100);
	//MountGame(gLua, "", -2114);

	return 0;
}

int Start(lua_State *L)
{
	ILuaInterface *gLua = Lua();

	gLua->SetGlobal("MountOtherGame", MountOtherGame);
	gLua->SetGlobal("IsGameMounted", IsGameMounted);
	return 0;
}

int Close(lua_State *L)
{
	return 0;
}